@@ -1,4 +1,5 @@
 """Models for Circuit Maintenance."""
+import hashlib
 from django.core.exceptions import ValidationError
 from django.db import models
 from django.urls import reverse
@@ -7,11 +8,7 @@ from nautobot.extras.utils import extras_features
 from nautobot.circuits.models import Circuit, Provider
 from nautobot.core.models.generics import PrimaryModel, OrganizationalModel
 
-from .choices import (
-    CircuitImpactChoices,
-    CircuitMaintenanceStatusChoices,
-    NoteLevelChoices,
-)
+from .choices import CircuitImpactChoices, CircuitMaintenanceStatusChoices, NoteLevelChoices
 
 
 @extras_features(
@@ -153,6 +150,47 @@ class Note(OrganizationalModel):
         return (self.maintenance, self.title, self.level, self.comment, self.date)
 
 
+@extras_features(
+    "custom_fields",
+    "custom_links",
+    "custom_validators",
+    "export_templates",
+    "relationships",
+    "webhooks",
+)
+class NotificationSource(OrganizationalModel):
+    """Model for Notification Source configuration."""
+
+    name = models.CharField(
+        max_length=100,
+        unique=True,
+        help_text="Notification Source Name as defined in configuration file.",
+    )
+    slug = models.SlugField(max_length=100, unique=True)
+    providers = models.ManyToManyField(
+        Provider,
+        help_text="The Provider(s) that this Notification Source applies to.",
+        blank=True,
+    )
+
+    csv_headers = ["name", "slug", "providers"]
+
+    class Meta:  # noqa: D106 "Missing docstring in public nested class"
+        ordering = ["name"]
+
+    def __str__(self):
+        """String value for HTML rendering."""
+        return f"{self.name}"
+
+    def get_absolute_url(self):
+        """Returns reverse loop up URL."""
+        return reverse("plugins:nautobot_circuit_maintenance:notificationsource", args=[self.slug])
+
+    def to_csv(self):
+        """Return fields for bulk view."""
+        return (self.name, self.slug, self.providers)
+
+
 @extras_features(
     "custom_fields",
     "custom_links",
@@ -164,21 +202,23 @@ class Note(OrganizationalModel):
 class RawNotification(OrganizationalModel):
     """Model for maintenance notifications in raw format."""
 
-    raw = models.TextField(unique=True)
+    raw = models.TextField()
+    _raw_md5 = models.TextField(unique=True)
     subject = models.CharField(max_length=200)
     provider = models.ForeignKey(Provider, on_delete=models.CASCADE, default=None)
-    sender = models.CharField(max_length=200)
-    source = models.CharField(
-        max_length=50,
-        null=True,
-        blank=True,
-    )
+    sender = models.CharField(max_length=200, default="", null=True, blank=True)
+    source = models.ForeignKey(NotificationSource, on_delete=models.SET_NULL, null=True)
     parsed = models.BooleanField(default=False, null=True, blank=True)
     date = models.DateTimeField(default=now)
 
     class Meta:  # noqa: D106 "Missing docstring in public nested class"
         ordering = ["date"]
 
+    def save(self, *args, **kwargs):
+        """Custom save for RawNotification."""
+        self._raw_md5 = hashlib.md5(self.raw).hexdigest()  # nosec
+        super().save(*args, **kwargs)
+
     def __str__(self):
         """String value for HTML rendering."""
         return f"{self.subject}"
@@ -222,44 +262,3 @@ class ParsedNotification(OrganizationalModel):
     def to_csv(self):
         """Return fields for bulk view."""
         return (self.maintenance, self.raw_notification, self.json, self.date)
-
-
-@extras_features(
-    "custom_fields",
-    "custom_links",
-    "custom_validators",
-    "export_templates",
-    "relationships",
-    "webhooks",
-)
-class NotificationSource(OrganizationalModel):
-    """Model for Notification Source configuration."""
-
-    name = models.CharField(
-        max_length=100,
-        unique=True,
-        help_text="Notification Source Name as defined in configuration file.",
-    )
-    slug = models.SlugField(max_length=100, unique=True)
-    providers = models.ManyToManyField(
-        Provider,
-        help_text="The Provider(s) that this Notification Source applies to.",
-        blank=True,
-    )
-
-    csv_headers = ["name", "slug", "providers"]
-
-    class Meta:  # noqa: D106 "Missing docstring in public nested class"
-        ordering = ["name"]
-
-    def __str__(self):
-        """String value for HTML rendering."""
-        return f"{self.name}"
-
-    def get_absolute_url(self):
-        """Returns reverse loop up URL."""
-        return reverse("plugins:nautobot_circuit_maintenance:notificationsource", args=[self.slug])
-
-    def to_csv(self):
-        """Return fields for bulk view."""
-        return (self.name, self.slug, self.providers)
