@@ -10,25 +10,28 @@ menu_items = (
         buttons=(
             PluginMenuButton(
                 link="plugins:nautobot_circuit_maintenance:circuitmaintenance_add",
-                title="List Circuit Maintenances",
+                title="Add Circuit Maintenance",
                 icon_class="mdi mdi-plus-thick",
                 color=ButtonColorChoices.GREEN,
                 permissions=["nautobot_circuit_maintenance.add_circuitmaintenance"],
             ),
+            PluginMenuButton(
+                link="plugins:nautobot_circuit_maintenance:circuitmaintenance_job",
+                title="Job to update Circuit Maintenance",
+                icon_class="mdi mdi-database-refresh",
+                color=ButtonColorChoices.GREEN,
+                permissions=["nautobot_circuit_maintenance.add_circuitmaintenance"],
+            ),
         ),
     ),
     PluginMenuItem(
         link="plugins:nautobot_circuit_maintenance:rawnotification_list",
         link_text="Notifications",
         permissions=["nautobot_circuit_maintenance.view_circuitmaintenance"],
-        buttons=(
-            PluginMenuButton(
-                link="plugins:nautobot_circuit_maintenance:notificationsource_list",
-                title="Notification Sources",
-                icon_class="mdi mdi-tune",
-                color=ButtonColorChoices.YELLOW,
-                permissions=["nautobot_circuit_maintenance.add_circuitmaintenance"],
-            ),
-        ),
+    ),
+    PluginMenuItem(
+        link="plugins:nautobot_circuit_maintenance:notificationsource_list",
+        link_text="Notification Sources",
+        permissions=["nautobot_circuit_maintenance.notificationsource_list"],
     ),
 )
